==> 4.5.0.tar.gz: terraform-provider-decort/internal/service/cloudbroker/extnet/utility_extnet_resource.go <==
/*
Copyright (c) 2019-2023 Digital Energy Cloud Solutions LLC. All Rights Reserved.
Authors:
Petr Krutov, <petr.krutov@digitalenergy.online>
Stanislav Solovev, <spsolovev@digitalenergy.online>
Kasim Baybikov, <kmbaybikov@basistech.ru>
Tim Tkachev, <tvtkachev@basistech.ru>

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

/*
Terraform DECORT provider - manage resources provided by DECORT (Digital Energy Cloud
Orchestration Technology) with Terraform by Hashicorp.

Source code: https://repository.basistech.ru/BASIS/terraform-provider-decort

Please see README.md to learn where to place source code so that it
builds seamlessly.

Documentation: https://repository.basistech.ru/BASIS/terraform-provider-decort/wiki
*/

package extnet

import (
	"context"

	"github.com/hashicorp/terraform-plugin-sdk/v2/helper/schema"
	log "github.com/sirupsen/logrus"
	"repository.basistech.ru/BASIS/decort-golang-sdk/pkg/cloudbroker/extnet"
	"repository.basistech.ru/BASIS/terraform-provider-decort/internal/controller"
	"repository.basistech.ru/BASIS/terraform-provider-decort/internal/service/cloudbroker/ic"
	"repository.basistech.ru/BASIS/terraform-provider-decort/internal/status"
)

func handleExcludedIPsUpdate(ctx context.Context, d *schema.ResourceData, c *controller.ControllerCfg, recNet *extnet.RecordExtNet) error {
	old_set, new_set := d.GetChange("excluded_ips")

	detach_set := old_set.(*schema.Set).Difference(new_set.(*schema.Set))
	if detach_set.Len() > 0 {
		ips := make([]string, 0)
		for _, detach_ip := range detach_set.List() {
			ips = append(ips, detach_ip.(string))
		}

		log.Debugf("cloudbroker: removing %d IP address(es) from excluded list", detach_set.Len())
		req := extnet.IPsIncludeRequest{
			NetID: recNet.ID,
			IPs:   ips,
		}

		_, err := c.CloudBroker().ExtNet().IPsInclude(ctx, req)
		if err != nil {
			return err
		}
	}

	attach_set := new_set.(*schema.Set).Difference(old_set.(*schema.Set))
	if attach_set.Len() > 0 {
		ips := make([]string, 0)
		for _, attach_ip := range attach_set.List() {
			ips = append(ips, attach_ip.(string))
		}

		log.Debugf("cloudbroker: excluding %d IP address(es) from extnet with id %d", attach_set.Len(), recNet.ID)
		req := extnet.IPsExcludeRequest{
			NetID: recNet.ID,
			IPs:   ips,
		}

		_, err := c.CloudBroker().ExtNet().IPsExclude(ctx, req)
		if err != nil {
			return err
		}
	}

	return nil
}

func handleSetDefault(ctx context.Context, d *schema.ResourceData, c *controller.ControllerCfg, recNet *extnet.RecordExtNet) error {
	set_default := d.Get("set_default").(bool)
	if set_default && !recNet.Default {
		req := extnet.SetDefaultRequest{
			NetID: recNet.ID,
		}

		_, err := c.CloudBroker().ExtNet().SetDefault(ctx, req)
		if err != nil {
			return err
		}
	}

	return nil
}

func handleBasicUpdate(ctx context.Context, d *schema.ResourceData, c *controller.ControllerCfg, recNet *extnet.RecordExtNet) error {
	doBasicUpdate := false
	basiUpdateReq := extnet.UpdateRequest{NetID: recNet.ID}

	if d.HasChange("name") {
		basiUpdateReq.Name = d.Get("name").(string)
		doBasicUpdate = true
	}
	if d.HasChange("desc") {
		basiUpdateReq.Description = d.Get("desc").(string)
		doBasicUpdate = true
	}

	if doBasicUpdate {
		_, err := c.CloudBroker().ExtNet().Update(ctx, basiUpdateReq)
		if err != nil {
			return err
		}
	}

	return nil
}

func handleEnableUpdate(ctx context.Context, d *schema.ResourceData, c *controller.ControllerCfg, recNet *extnet.RecordExtNet) error {
	enable := d.Get("enable").(bool)
	if enable {
		if recNet.Status == status.Disabled {
			req := extnet.EnableRequest{NetID: recNet.ID}
			_, err := c.CloudBroker().ExtNet().Enable(ctx, req)
			if err != nil {
				return err
			}
		}
	} else {
		if recNet.Status == status.Enabled {
			req := extnet.DisableRequest{NetID: recNet.ID}
			_, err := c.CloudBroker().ExtNet().Disable(ctx, req)
			if err != nil {
				return err
			}
		}
	}

	return nil
}

func handleDefaultQOSUpdate(ctx context.Context, d *schema.ResourceData, c *controller.ControllerCfg, recNet *extnet.RecordExtNet) error {
	qos := d.Get("default_qos").([]interface{})[0].(map[string]interface{})

	req := extnet.DefaultQOSUpdateRequest{
		NetID:        recNet.ID,
		IngressRate:  uint64(qos["in_rate"].(int)),
		IngressBurst: uint64(qos["in_burst"].(int)),
		EgressRate:   uint64(qos["e_rate"].(int)),
	}

	_, err := c.CloudBroker().ExtNet().DefaultQOSUpdate(ctx, req)
	if err != nil {
		return err
	}

	return nil
}

func handleNTPUpdate(ctx context.Context, d *schema.ResourceData, c *controller.ControllerCfg, recNet *extnet.RecordExtNet) error {
	changed_list := d.Get("ntp").([]interface{})

	ntp_list := make([]string, 0)
	for _, ntp_address := range changed_list {
		ntp_list = append(ntp_list, ntp_address.(string))
	}

	req := extnet.NTPApplyRequest{
		NetID:   recNet.ID,
		NTPList: ntp_list,
	}

	_, err := c.CloudBroker().ExtNet().NTPApply(ctx, req)
	if err != nil {
		return err
	}

	return nil
}

func handleDNSUpdate(ctx context.Context, d *schema.ResourceData, c *controller.ControllerCfg, recNet *extnet.RecordExtNet) error {
	changed_list := d.Get("dns").([]interface{})

	dns_list := make([]string, 0)
	for _, dns_address := range changed_list {
		dns_list = append(dns_list, dns_address.(string))
	}

	req := extnet.DNSApplyRequest{
		NetID:   recNet.ID,
		DNSList: dns_list,
	}

	_, err := c.CloudBroker().ExtNet().DNSApply(ctx, req)
	if err != nil {
		return err
	}

	return nil
}

func handleExcludedIPsRangeUpdate(ctx context.Context, d *schema.ResourceData, c *controller.ControllerCfg, recNet *extnet.RecordExtNet) error {
	old_set, new_set := d.GetChange("excluded_ips_range")

	detach_set := old_set.(*schema.Set).Difference(new_set.(*schema.Set))
	if detach_set.Len() > 0 {
		for _, detach_ip_range := range detach_set.List() {

			log.Debugf("cloudbroker: removing range of IP addreses from excluded range list")

			req := extnet.IPsIncludeRangeRequest{
				NetID:   recNet.ID,
				IPStart: detach_ip_range.(map[string]interface{})["ip_start"].(string),
				IPEnd:   detach_ip_range.(map[string]interface{})["ip_end"].(string),
			}

			_, err := c.CloudBroker().ExtNet().IPsIncludeRange(ctx, req)
			if err != nil {
				return err
			}
		}
	}
	attach_set := new_set.(*schema.Set).Difference(old_set.(*schema.Set))
	if attach_set.Len() > 0 {
		for _, attach_ip_range := range attach_set.List() {

			log.Debugf("cloudbroker: excluding range of IP addreses from excluded range list")

			req := extnet.IPsExcludeRangeRequest{
				NetID:   recNet.ID,
				IPStart: attach_ip_range.(map[string]interface{})["ip_start"].(string),
				IPEnd:   attach_ip_range.(map[string]interface{})["ip_end"].(string),
			}

			_, err := c.CloudBroker().ExtNet().IPsExcludeRange(ctx, req)
			if err != nil {
				return err
			}
		}
	}
	return nil
}

func handleSharedWithUpdate(ctx context.Context, d *schema.ResourceData, c *controller.ControllerCfg) error {
	deletedIds := make([]uint64, 0)
	addedIds := make([]uint64, 0)

	oldAccountIds, newAccountIds := d.GetChange("shared_with")
	oldAccountIdsSlice := oldAccountIds.([]interface{})
	newAccountIdsSlice := newAccountIds.([]interface{})

	for _, el := range oldAccountIdsSlice {
		if !isContainsIds(newAccountIdsSlice, el) {
			convertedEl := uint64(el.(int))
			deletedIds = append(deletedIds, convertedEl)
		}
	}

	for _, el := range newAccountIdsSlice {
		if !isContainsIds(oldAccountIdsSlice, el) {
			convertedEl := uint64(el.(int))
			addedIds = append(addedIds, convertedEl)
		}
	}

	if len(deletedIds) > 0 {
		for _, accountId := range deletedIds {
			req := extnet.AccessRemoveRequest{
				NetID:     uint64(d.Get("extnet_id").(int)),
				AccountID: accountId,
			}

			_, err := c.CloudBroker().ExtNet().AccessRemove(ctx, req)
			if err != nil {
				return err
			}
		}
	}

	if len(addedIds) > 0 {
		for _, accountId := range addedIds {
			req := extnet.AccessAddRequest{
				NetID:     uint64(d.Get("extnet_id").(int)),
				AccountID: accountId,
			}

			_, err := c.CloudBroker().ExtNet().AccessAdd(ctx, req)
			if err != nil {
				return err
			}
		}

	}

	return nil
}

func handleVirtualUpdate(ctx context.Context, d *schema.ResourceData, c *controller.ControllerCfg, recNet *extnet.RecordExtNet) error {
	virtualOld, virtualNew := d.GetChange("virtual")

	if virtualOld == false && virtualNew == true {
		req := extnet.DeviceRemoveRequest{NetID: recNet.ID}
		_, err := c.CloudBroker().ExtNet().DeviceRemove(ctx, req)
		if err != nil {
			return err
		}

	} else if virtualOld == true && virtualNew == false {
		req := extnet.DeviceDeployRequest{NetID: recNet.ID}
		_, err := c.CloudBroker().ExtNet().DeviceDeploy(ctx, req)
		if err != nil {
			return err
		}
	}
	return nil
}

func handleRestartUpdate(ctx context.Context, d *schema.ResourceData, c *controller.ControllerCfg, recNet *extnet.RecordExtNet) error {
	restartOld, restartNew := d.GetChange("restart")

	if restartOld == false && restartNew == true {
		req := extnet.DeviceRestartRequest{NetID: recNet.ID}
		_, err := c.CloudBroker().ExtNet().DeviceRestart(ctx, req)
		if err != nil {
			return err
		}
	}
	return nil
}

func handleMigrateUpdate(ctx context.Context, d *schema.ResourceData, c *controller.ControllerCfg, recNet *extnet.RecordExtNet) error {
	stackId := uint64(d.Get("migrate").(int))

	if err := ic.ExistStack(ctx, stackId, c); err != nil {
		return err
	}

	req := extnet.DeviceMigrateRequest{
		NetID:   recNet.ID,
		StackID: stackId,
	}

	_, err := c.CloudBroker().ExtNet().DeviceMigrate(ctx, req)
	if err != nil {
		return err
	}

	return nil
}

==> 4.7.4.tar.gz: terraform-provider-decort/internal/service/cloudbroker/extnet/utility_extnet_resource.go <==
/*
Copyright (c) 2019-2023 Digital Energy Cloud Solutions LLC. All Rights Reserved.
Authors:
Petr Krutov, <petr.krutov@digitalenergy.online>
Stanislav Solovev, <spsolovev@digitalenergy.online>
Kasim Baybikov, <kmbaybikov@basistech.ru>
Tim Tkachev, <tvtkachev@basistech.ru>

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

/*
Terraform DECORT provider - manage resources provided by DECORT (Digital Energy Cloud
Orchestration Technology) with Terraform by Hashicorp.

Source code: https://repository.basistech.ru/BASIS/terraform-provider-decort

Please see README.md to learn where to place source code so that it
builds seamlessly.

Documentation: https://repository.basistech.ru/BASIS/terraform-provider-decort/wiki
*/

package extnet

import (
	"context"

	"github.com/hashicorp/terraform-plugin-sdk/v2/helper/schema"
	log "github.com/sirupsen/logrus"
	"repository.basistech.ru/BASIS/decort-golang-sdk/pkg/cloudbroker/extnet"
	"repository.basistech.ru/BASIS/terraform-provider-decort/internal/controller"
	"repository.basistech.ru/BASIS/terraform-provider-decort/internal/service/cloudbroker/ic"
	"repository.basistech.ru/BASIS/terraform-provider-decort/internal/status"
)

func handleExcludedIPsUpdate(ctx context.Context, d *schema.ResourceData, c *controller.ControllerCfg, recNet *extnet.RecordExtNet) error {
	old_set, new_set := d.GetChange("excluded_ips")

	detach_set := old_set.(*schema.Set).Difference(new_set.(*schema.Set))
	if detach_set.Len() > 0 {
		ips := make([]string, 0)
		for _, detach_ip := range detach_set.List() {
			ips = append(ips, detach_ip.(string))
		}

		log.Debugf("cloudbroker: removing %d IP address(es) from excluded list", detach_set.Len())
		req := extnet.IPsIncludeRequest{
			NetID: recNet.ID,
			IPs:   ips,
		}

		_, err := c.CloudBroker().ExtNet().IPsInclude(ctx, req)
		if err != nil {
			return err
		}
	}

	attach_set := new_set.(*schema.Set).Difference(old_set.(*schema.Set))
	if attach_set.Len() > 0 {
		ips := make([]string, 0)
		for _, attach_ip := range attach_set.List() {
			ips = append(ips, attach_ip.(string))
		}

		log.Debugf("cloudbroker: excluding %d IP address(es) from extnet with id %d", attach_set.Len(), recNet.ID)
		req := extnet.IPsExcludeRequest{
			NetID: recNet.ID,
			IPs:   ips,
		}

		_, err := c.CloudBroker().ExtNet().IPsExclude(ctx, req)
		if err != nil {
			return err
		}
	}

	return nil
}

func handleSetDefault(ctx context.Context, d *schema.ResourceData, c *controller.ControllerCfg, recNet *extnet.RecordExtNet) error {
	set_default := d.Get("set_default").(bool)
	if set_default && !recNet.Default {
		req := extnet.SetDefaultRequest{
			NetID: recNet.ID,
		}

		_, err := c.CloudBroker().ExtNet().SetDefault(ctx, req)
		if err != nil {
			return err
		}
	}

	return nil
}

func handleBasicUpdate(ctx context.Context, d *schema.ResourceData, c *controller.ControllerCfg, recNet *extnet.RecordExtNet) error {
	doBasicUpdate := false
	basiUpdateReq := extnet.UpdateRequest{NetID: recNet.ID}

	if d.HasChange("name") {
		basiUpdateReq.Name = d.Get("name").(string)
		doBasicUpdate = true
	}
	if d.HasChange("desc") {
		basiUpdateReq.Description = d.Get("desc").(string)
		doBasicUpdate = true
	}

	if doBasicUpdate {
		_, err := c.CloudBroker().ExtNet().Update(ctx, basiUpdateReq)
		if err != nil {
			return err
		}
	}

	return nil
}

func handleEnableUpdate(ctx context.Context, d *schema.ResourceData, c *controller.ControllerCfg, recNet *extnet.RecordExtNet) error {
	enable := d.Get("enable").(bool)
	if enable {
		if recNet.Status == status.Disabled {
			req := extnet.EnableRequest{NetID: recNet.ID}
			_, err := c.CloudBroker().ExtNet().Enable(ctx, req)
			if err != nil {
				return err
			}
		}
	} else {
		if recNet.Status == status.Enabled {
			req := extnet.DisableRequest{NetID: recNet.ID}
			_, err := c.CloudBroker().ExtNet().Disable(ctx, req)
			if err != nil {
				return err
			}
		}
	}

	return nil
}

func handleDefaultQOSUpdate(ctx context.Context, d *schema.ResourceData, c *controller.ControllerCfg, recNet *extnet.RecordExtNet) error {
	qos := d.Get("default_qos").([]interface{})[0].(map[string]interface{})

	req := extnet.DefaultQOSUpdateRequest{
		NetID:        recNet.ID,
		IngressRate:  uint64(qos["in_rate"].(int)),
		IngressBurst: uint64(qos["in_burst"].(int)),
		EgressRate:   uint64(qos["e_rate"].(int)),
	}

	_, err := c.CloudBroker().ExtNet().DefaultQOSUpdate(ctx, req)
	if err != nil {
		return err
	}

	return nil
}

func handleNTPUpdate(ctx context.Context, d *schema.ResourceData, c *controller.ControllerCfg, recNet *extnet.RecordExtNet) error {
	changed_list := d.Get("ntp").([]interface{})

	ntp_list := make([]string, 0)
	for _, ntp_address := range changed_list {
		ntp_list = append(ntp_list, ntp_address.(string))
	}

	req := extnet.NTPApplyRequest{
		NetID:   recNet.ID,
		NTPList: ntp_list,
	}

	_, err := c.CloudBroker().ExtNet().NTPApply(ctx, req)
	if err != nil {
		return err
	}

	return nil
}

func handleDNSUpdate(ctx context.Context, d *schema.ResourceData, c *controller.ControllerCfg, recNet *extnet.RecordExtNet) error {
	changed_list := d.Get("dns").([]interface{})

	dns_list := make([]string, 0)
	for _, dns_address := range changed_list {
		dns_list = append(dns_list, dns_address.(string))
	}

	req := extnet.DNSApplyRequest{
		NetID:   recNet.ID,
		DNSList: dns_list,
	}

	_, err := c.CloudBroker().ExtNet().DNSApply(ctx, req)
	if err != nil {
		return err
	}

	return nil
}

func handleExcludedIPsRangeUpdate(ctx context.Context, d *schema.ResourceData, c *controller.ControllerCfg, recNet *extnet.RecordExtNet) error {
	old_set, new_set := d.GetChange("excluded_ips_range")

	detach_set := old_set.(*schema.Set).Difference(new_set.(*schema.Set))
	if detach_set.Len() > 0 {
		for _, detach_ip_range := range detach_set.List() {

			log.Debugf("cloudbroker: removing range of IP addreses from excluded range list")

			req := extnet.IPsIncludeRangeRequest{
				NetID:   recNet.ID,
				IPStart: detach_ip_range.(map[string]interface{})["ip_start"].(string),
				IPEnd:   detach_ip_range.(map[string]interface{})["ip_end"].(string),
			}

			_, err := c.CloudBroker().ExtNet().IPsIncludeRange(ctx, req)
			if err != nil {
				return err
			}
		}
	}
	attach_set := new_set.(*schema.Set).Difference(old_set.(*schema.Set))
	if attach_set.Len() > 0 {
		for _, attach_ip_range := range attach_set.List() {

			log.Debugf("cloudbroker: excluding range of IP addreses from excluded range list")

			req := extnet.IPsExcludeRangeRequest{
				NetID:   recNet.ID,
				IPStart: attach_ip_range.(map[string]interface{})["ip_start"].(string),
				IPEnd:   attach_ip_range.(map[string]interface{})["ip_end"].(string),
			}

			_, err := c.CloudBroker().ExtNet().IPsExcludeRange(ctx, req)
			if err != nil {
				return err
			}
		}
	}
	return nil
}

func handleSharedWithUpdate(ctx context.Context, d *schema.ResourceData, c *controller.ControllerCfg) error {
	oldSet, newSet := d.GetChange("shared_with")

	deletedAccountIds := (oldSet.(*schema.Set).Difference(newSet.(*schema.Set))).List()
	if len(deletedAccountIds) > 0 {
		for _, accountIdInterface := range deletedAccountIds {
			req := extnet.AccessRemoveRequest{
				NetID:     uint64(d.Get("extnet_id").(int)),
				AccountID: uint64(accountIdInterface.(int)),
			}

			_, err := c.CloudBroker().ExtNet().AccessRemove(ctx, req)
			if err != nil {
				return err
			}
		}
	}

	addedAccountIds := (newSet.(*schema.Set).Difference(oldSet.(*schema.Set))).List()
	if len(addedAccountIds) > 0 {
		for _, accountIdInterface := range addedAccountIds {
			req := extnet.AccessAddRequest{
				NetID:     uint64(d.Get("extnet_id").(int)),
				AccountID: uint64(accountIdInterface.(int)),
			}

			_, err := c.CloudBroker().ExtNet().AccessAdd(ctx, req)
			if err != nil {
				return err
			}
		}
	}

	return nil
}

func handleVirtualUpdate(ctx context.Context, d *schema.ResourceData, c *controller.ControllerCfg, recNet *extnet.RecordExtNet) error {
	virtualOld, virtualNew := d.GetChange("virtual")

	if !virtualOld.(bool) && virtualNew.(bool) {
		req := extnet.DeviceRemoveRequest{NetID: recNet.ID}
		_, err := c.CloudBroker().ExtNet().DeviceRemove(ctx, req)
		if err != nil {
			return err
		}

	} else if virtualOld.(bool) && !virtualNew.(bool) {
		req := extnet.DeviceDeployRequest{NetID: recNet.ID}
		_, err := c.CloudBroker().ExtNet().DeviceDeploy(ctx, req)
		if err != nil {
			return err
		}
	}
	return nil
}

func handleRestartUpdate(ctx context.Context, d *schema.ResourceData, c *controller.ControllerCfg, recNet *extnet.RecordExtNet) error {
	restartOld, restartNew := d.GetChange("restart")

	if !restartOld.(bool) && restartNew.(bool) {
		req := extnet.DeviceRestartRequest{NetID: recNet.ID}
		_, err := c.CloudBroker().ExtNet().DeviceRestart(ctx, req)
		if err != nil {
			return err
		}
	}
	return nil
}

func handleMigrateUpdate(ctx context.Context, d *schema.ResourceData, c *controller.ControllerCfg, recNet *extnet.RecordExtNet) error {
	stackId := uint64(d.Get("migrate").(int))

	if err := ic.ExistStack(ctx, stackId, c); err != nil {
		return err
	}

	req := extnet.DeviceMigrateRequest{
		NetID:   recNet.ID,
		StackID: stackId,
	}

	_, err := c.CloudBroker().ExtNet().DeviceMigrate(ctx, req)
	if err != nil {
		return err
	}

	return nil
}
